<?php
// $Id:

define ('CFORGE_PROFILE_PATH', 'profiles/cforge/');

/**
 * Return an array of the modules to be enabled when this profile is installed.
 *
 * @return
 *   An array of modules to enable.
 */
function cforge_profile_modules() {
  //what happens if these mods aren't available?
  $core = array('contact', 'comment', 'dblog', 'help', 'menu', 'path', 'php', 'profile', 'search', 'taxonomy', 'token', 'update', 'upload');
  $common = array('captcha', 'content', 'date_api', 'image', 'imageapi', 'imageapi_gd', 'imagecache', 'jquery_ui', 'usertabs', 'views');
  $other = array('backup_migrate', 'date', 'date_popup', 'hierarchical_select', 'hs_taxonomy', 'image_gallery', 'jquerymenu', 'mcapi', 'mc_webforms', 'simplenews', 'simplenews_action');
  return array_merge($core, $common, $other);
}

/**
 * Return a description of the profile for the initial installation screen.
 *
 * @return
 *   An array with keys 'name' and 'description' describing this profile,
 *   and optional 'language' to override the language selection for
 *   language-specific profiles.
 */
function cforge_profile_details() {
  return array(
    'name' => 'Community Forge',
    'description' => st('Full community accounting site similar to demo.communityforge.net')
  );
}

/**
 * Return a list of tasks that this profile supports.
 *
 * @return
 *   A keyed array of tasks the profile will perform during
 *   the final stage. The keys of the array will be used internally,
 *   while the values will be displayed to the user in the installer
 *   task list.
 */
function cforge_profile_task_list() {
  return array(
    'cforge_install' => st('Cforge options'),
  );
}


/**
 * Implementation of hook_form_alter().
 *
 * Allows the profile to alter the site-configuration form. This is
 * called through custom invocation, so $form_state is not populated.
 */
function cforge_form_alter(&$form, $form_state, $form_id) {
  if ($form_id != 'install_configure') return;

  if (is_file(CFORGE_PROFILE_PATH . 'matslats.inc')) {
    include_once(CFORGE_PROFILE_PATH . 'matslats.inc');
    $form['site_information']['site_name']['#default_value'] = $_SERVER['SERVER_NAME'];
    $form['admin_account']['account']['name']['#default_value'] = 'admin';
    $form['admin_account']['account']['name']['#type'] = 'hidden';
    $form['admin_account']['account']['pass']['#value'] = $cfpass;
    $form['admin_account']['account']['pass']['#type'] = 'hidden';
    $form['admin_account']['account']['mail']['#default_value'] = 'tim@communityforge.net';
    $form['admin_account']['account']['mail']['#type'] = 'hidden';
    $form['server_settings']['date_default_timezone']['#default_value'] = 3600;
  }
}

/**
 * implements install hook_tasks
 * see default.profile for more details
 */
//this is not being called with the declared tasks
function cforge_profile_tasks(&$task, $url) {
  if ($task == 'profile') {
    $task = 'cforge_install';
    global $language;
    if ($language->language != 'en' && module_exists('locale')) {
      //this needs to be done now before any menu items are created
      cforge_import_translations($language->language);
    }
    cforge_general_setup();
  }
  //deliver and process a form with the installation options
  if ($task == 'cforge_install') {
    if (function_exists('drush_get_option')) {
      //if aegir is in charge, we take the options via drush, not by a form submission
      cforge_profile_drush_process();
      drupal_set_message('Cforge install profile options will be handled by drush.');
    }
    else {
      drupal_set_message('Cforge install profile options will be handled normally');
      $output = drupal_get_form('cforge_install_form', $url);
      if (!variable_get('cforge_install_form_submitted', FALSE)) {
        drupal_set_title(st('Cforge installation options'));
        return $output;
      }
    }
    variable_del('cforge_install_form_submitted');
    $task = 'profile-finished';//this means we passed validation
  }
}

function cforge_import_translations($lan) {
  $po_file = CFORGE_PROFILE_PATH . 'translations/'.$lan.'.po';
  if (file_exists($po_file)) {
    $file = (object)array('filepath' => $po_file);
    include_once('includes/locale.inc');
    _locale_import_po($file, 'fr', LOCALE_IMPORT_OVERWRITE, 'default');
  }
  else {
    drupal_set_message('Failed to import po file', 'warning');
  }
}

/*
 * NOTICE!!
 *
 * Changes to this form need to be reflected in aegir_cforge.module
 */
function cforge_install_form($form_state = array(), $task = NULL) {
  $form['uid_login'] = array(
    '#type' => 'radios',
    '#title' => 'Log in',
    '#options' => array(0 => st('Username'), 1 => st('Unique number')),
    '#default_value' => 0,
  );
  $form['autocategorise'] = array(
    '#type' => 'checkbox',
    '#title' => st('Use automatic categorisation'),
    '#default_value' => 0,
  );
  $form['imports'] = array(
    '#type' => 'checkbox',
    '#title' => st('Enable data import tools'),
    '#description' => st("If your SEL has more than, say 25 members, you may want to import your data via csv."),
    '#default_value' => 0,
  );
  if (strpos(variable_get('site_mail', 'blah'), 'communityforge')) {
    $form['site_mail'] = array(
      '#type' => 'textfield',
      '#title' => st('Site mail'),
      '#default_value' => '',
      '#lines' => 3,
    );
  }
  $form['hours'] = array(
    '#type' => 'checkbox',
    '#title' => st('Preset currency to hours'),
    '#default_value' => '',
  );
  $vocabs = scandir(CFORGE_PROFILE_PATH . 'taxonomy_import/1');//1 is the main vocab AFTER cforge_custom is installed
  unset($vocabs[0], $vocabs[1]);
  $vocabs = array_filter($vocabs, '_cforge_filter_txtfile');
  $form['vocab_1'] = array(
    '#type' => 'select',
    '#title' => st('Marketplace vocabulary'),
    '#description' => st('Optionally import terms into your main vocabulary'),
    '#options' => array_merge(array('<none>'), drupal_map_assoc($vocabs))
  );

  $form['intersel'] = array(
    '#type' => 'checkbox',
    '#title' => st('Create Intersel account'),
    '#default_value' => 0,
  );
  $form['rds'] = array(
    '#type' => 'checkbox',
    '#title' => st('Create Route des SEL account'),
    '#default_value' => 0,
  );

  $form['local_admin'] = array(
    '#type' => 'fieldset',
    '#title' => st('local administrator'),
    '#description' => st('Creates a administrator user with insufficient permission to break the site (optional)'),
    '#collapsible' => FALSE,
  );
  $form['local_admin']['profile_firstname'] = array('#type' => 'textfield',
    '#title' => st('First name'),
    '#maxlength' => USERNAME_MAX_LENGTH/2,
    '#description' => st('Spaces are allowed; punctuation is not allowed except for periods, hyphens, and underscores.'),
    '#weight' => -10,
  );
  $form['local_admin']['profile_familyname'] = array('#type' => 'textfield',
    '#title' => st('Family name'),
    '#maxlength' => USERNAME_MAX_LENGTH/2,
    '#description' => st('Spaces are allowed; punctuation is not allowed except for periods, hyphens, and underscores.'),
    '#weight' => -9,
  );

  $form['local_admin']['mail'] = array('#type' => 'textfield',
    '#title' => st('E-mail address'),
    '#maxlength' => EMAIL_MAX_LENGTH,
    '#description' => st('All e-mails from the system will be sent to this address. The e-mail address is not made public and will only be used if you wish to receive a new password or wish to receive certain news or notifications by e-mail.'),
    '#weight' => -5,
  );
  $form['local_admin']['pass'] = array(
    '#type' => 'password_confirm',
    '#required' => FALSE,
    '#size' => 25,
    '#weight' => 0,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'submit'
  );
  $form['#action'] = $url;
  $form['#redirect'] = FALSE;
  return $form;
}


/*
 * form submit handler for 2nd stage of installation, not using aegir
 */
function cforge_install_form_submit($form, $form_state) {
  variable_set('cforge_install_form_submitted', TRUE);
  global $language;
  if (is_file(CFORGE_PROFILE_PATH . 'matslats.inc')) {
    include_once(CFORGE_PROFILE_PATH . 'matslats.inc');
  }

  $values = $form_state['values'];
  if (!empty($values['site_mail'])) {
    variable_set('site_mail', $values['site_mail']);
  }
  $modules = array('community_tasks', 'calendar', 'cforge_docs', 'imagecache_profiles', 'jcalendar', 'mc_display', 'mc_multiple', 'mc_signatures', 'mc_webform_1stparty', 'mc_user_limits', 'offers_wants', 'role_expire', 'views_bonus_export');
  drupal_install_modules($modules);

  //optional modules.
  $modules = array();
  if ($values['autocategorise']) {
    $modules[] = 'autocategorise';
    //autocategorise settings
    variable_set('autocategorise_vids', array(variable_get('offers_wants_cat_vid', 1) => TRUE));
  }
  if ($values['uid_login']) {
    $modules[] = 'uid_login';
  }
  if ($values['imports']) {
    $modules[] = 'user_import';
    $modules[] = 'offers_wants_import';
    $modules[] = 'mcapi_import';
  }
  if (module_exists('locale')) {
    $modules[] = 'l10n_client';
    $modules[] = 'l10n_update';
  }
  drupal_install_modules($modules);

  //smtp settings
  if (file_exists('sites/all/modules/smtp/smtp.module') && $smtp_password) {
    drupal_install_modules(array('smtp'));
    variable_set('smtp_debugging', 0);
    variable_set('smtp_from', variable_get('site_mail', 'matslats@communityforge.net'));
    variable_set('smtp_fromname', variable_get('site_name', 'Community Forge'));
    variable_set('smtp_host', 'smtp.gmail.com');
    //variable_set('smtp_hostbackup');
    variable_set('smtp_library', drupal_get_filename('module', 'smtp'));
    variable_set('smtp_on', TRUE);
    variable_set('smtp_port', '587');
    variable_set('smtp_protocol', 'tls');
    variable_set('smtp_username', 'matslats@communityforge.net');
  }

  //mcapi settings
  variable_set('mc_subtypes', array(INCOMING_SIGNED => st('I am asking'), OUTGOING_SIGNED => st('I am giving')));
  variable_set('mc_notification_default', 1);
  variable_set('mc_notification_throttle_default', 2);
  variable_set('mc_sig_notification_default', 1);
  variable_set('mc_sig_notification_option', 1);
  //These need to be translated to Englisn and moved to the .import files
  variable_set('mc_notification_subject', "Echange saisi sur !site avec !username");
  variable_set('mc_notification_body', "Bonjour !username,\n\nUn échange a était enregistré.");
  variable_set('mc_notification_update_body', 'Bonjour !username,\n\nCet échange saisi précédement avec vous a était modifié :\n!summary');
  variable_set('mc_notification_update_subject', "Modification d'échange sur !site");
  variable_set('mc_signatures_notification_subject', "Veuillez confirmer cet échange sur !site");
  variable_set('mc_signatures_notification_body', "Veuillez cliquer sur ce lien pour confirmer (et comptabiliser) l'échange : !sign_link");
  variable_set('mc_notification_option', FALSE);
  variable_set('mc_signatures_signoff_subject', st('Notification from !site'));
  variable_set('mc_signatures_signoff_body', 'Greetings

The following exchange was confirmed and your balance updated:
!summary

The team at !site');
  variable_set('mc_stats_exclude', '1');
  variable_set('mc_tabs_css', 'tab-wrapper');
  variable_set('mc_live_stats', TRUE);

  //preset the default currency
  //this could alternatively be done with a node_load() approach, assuming node/1
  $curr = db_fetch_object(db_query("SELECT * from {mc_currencies}"));
  $data = unserialize($curr->data);
  $data['subtype_bitmask'] = 12;
  if ($values['hours']) {
    $sub = 1;
    $min = -40;
    $max = 40;
    $curr->symbol = '@minus<strong>H</strong>@integer:@subdivision';
    $data['divisions'] = "0|00\n25|15 mins\n50|30 mins\n75|45 mins";
    db_query("UPDATE {node_revisions} SET title = '%s' WHERE nid = %d", st('Hours'), 1);
  }
  else {
    $sub = 0;
    $min = -100;
    $max = 100;
    $symbol = $curr->symbol;
  }
  db_query("UPDATE {mc_currencies} SET min = %f, max = %f, data = '%s', symbol = '%s', sub = %d WHERE nid = %d",
    array(
      $min, $max, serialize($data), $symbol, $sub, $curr->nid
    )
  );

  //offers_wants settings
  variable_set('offers_wants_expiry', TRUE);
  variable_set('proposition_has_body', TRUE);
  variable_set('offers_wants_proposition_wants', TRUE);
  variable_set('comment_proposition', TRUE);

  //community task settings
  db_query(
    "UPDATE {node_type} SET description = '%s' WHERE type = 'community_task'",
    array(':desc' => st('Allows you to: enter a to-do task; list the tasks for members to commit to them; remind the committed member on his list of contributions; assign a recognition point when the task is accomplished.'))
  );


  //backup_restore settings
  $nodata = array ('cache', 'cache_block', 'cache_content', 'cache_filter',  'cache_form', 'cache_hierarchical_select',
    'cache_menu', 'cache_page', 'cache_update', 'cache_views', 'cache_views_data', 'captcha_points', 'captcha_sessions',
    'flood',  'search_dataset', 'search_index', 'search_node_links', 'search_total', 'semaphore', 'sessions',
    'views_object_cache', 'watchdog'
  );
  $filters = array(
    'compression' => 'zip',
    'notify_success_enable' => 0,
    'notify_success_email' => 'admin@matslats.net',
    'notify_failure_enable' => 0,
    'notify_failure_email' => 'admin@matslats.net',
    'utils_site_offline' => 0,
    'utils_site_offline_message' => st('Community Forge has taken your site offline briefly for maintenance. Thank you for your patience.'),
    'exclude_tables' => array (),
    'nodata_tables' => drupal_map_assoc($nodata),
    'utils_lock_tables' => 1
  );
  db_query("UPDATE backup_migrate_profiles SET
      name = 'Cforge_backup', filename = 'cforge', append_timestamp = 1, timestamp_format = 'Y-m-d\\TH-i-s', filters = '%s'
      WHERE profile_id = 'default'", serialize($filters)
  );

  //hiearchical_select settings
  variable_set('taxonomy_hierarchical_select_1', TRUE);
  variable_get('hs_config_taxonomy-1', array(
    'config_id' => 'taxonomy-1',
    'save_lineage' => 1,
    'enforce_deepest' => 0,
    'entity_count' => 0,
    'require_entity' => 0,
    'resizable' => 1,
    'level_labels' => array(
      'status' => 1,
      'labels' => array(
        '0' => 'Concerne',
      )
    ),
    'dropbox' => array(
        'status' => 1,
        'title' => 'All selections',
        'limit' => 0,
        'reset_hs' => 1
      ),
    'editability' => array(
      'status' => 0,
      'item_types' => array(
        '0' => NULL
      ),
      'allowed_levels' => array(
        '0' => 1
      ),
      'allow_new_levels' => 0,
        'max_levels' => 1
      )
    )
  );
  
  //simplenews settings
  $term = array(
    'vid' => variable_get('simplenews_vid', 0),
    'name' => st('Governance'),
    'weight' => 0,
    'description' => st('A newsletter for the decision makers')
  );
  taxonomy_save_term($term);

  $loginstring = module_exists('uid_login') ? "User ID: !uid" : "Username : !username";
  //write the default emails according to whether uid_login is installed
  //for when the administrator creates an account
  variable_set('user_mail_register_admin_created_subject', "Your new account at !site");
  variable_set('user_mail_register_admin_created_body', st("!username,

A site administrator at !site has created an account for you. You may now log in to !login_uri using the following username and password:

@loginstring
password: !password

You may also log in by clicking on this link or copying and pasting it in your browser:

!login_url

This is a one-time login, so it can be used only once.

After logging in, you will be redirected to !edit_uri so you can change your password.

--  !site team", array('@loginstring' => $loginstring)));

  //for when the user creates an account
  variable_set('user_mail_pending_approval_subject', "Account details for !username at !site (pending admin approval)");
  variable_set('user_mail_pending_approval_body', "!username,

Thank you for registering at !site. Your application for an account is currently pending approval. Once it has been approved, you will receive another e-mail containing information about how to log in, set your password, and other details.

--  !site team");

  //for when an account is unblocked.
  variable_set('user_mail_status_activated_subject', "Account details for !username at !site (approved)");
  variable_set('user_mail_status_activated_body', "!username,

Your account at !site has been activated.

You may now log in by clicking on this link or copying and pasting it in your browser:

!login_url

This is a one-time login, so it can be used only once.

After logging in, you will be redirected to !edit_uri so you can change your password.

Once you have set your own password, you will be able to log in to !login_uri in the future using:

@loginstring", array('@loginstring' => $loginstring));

  if ($values['mail'] && $values['profile_firstname'] && $values['profile_firstname']) {
    //add the local admin account
    user_save(NULL, array(
      'name' => $values['profile_firstname'] .' '. $values['profile_familyname'],
      'pass' => $values['pass'],
      'mail' => $values['mail'],
      'status' => 1,
      'init' => $values['mail'],
      'profile_firstname' => $values['profile_firstname'],
      'profile_familyname' => $values['profile_familyname'],
      'roles' => drupal_map_assoc(array(3,4,5,6))
    ));
  }
  if ($values['rds']){
    user_save(NULL, array(
      'name' => 'Route des SEL',
      'pass' => user_password(8), // note: do not md5 the password
      'mail' => 'rds@communityforge.net',
      'status' => 1,
      'init' => 'rds@communityforge.net',
      'profile_firstname' => 'Route',
      'profile_familyname' => 'des SEL',
      'roles' => drupal_map_assoc(array(7))
    ));
  }
  if ($values['intersel']){
    $account = user_save(NULL, array(
      'name' => 'inter SEL',
      'pass' => user_password(8), // note: do not md5 the password
      'mail' => 'intersel@communityforge.net',
      'status' => 1,
      'init' => 'intersel@communityforge.net',
      'profile_firstname' => 'Inter',
      'profile_familyname' => 'SEL',
      'roles' => drupal_map_assoc(array(7))
    ));
    variable_set('intertrading_uid', $account->uid);
  }
  //now install cforge_custom
  db_query("UPDATE {system} SET weight = 50 WHERE name = 'cforge_custom';");
  drupal_install_modules(array('cforge_custom'));
  variable_set('cforge_report', 1);
  variable_set('cf_unlaunched', TRUE);
  variable_set('user_mail_password_reset_subject', st('New password for !site'));


  module_load_include('module', 'node');
  //retrieve the default content
  $path = CFORGE_PROFILE_PATH . 'content/'.$language->language;
  $level1 = scandir($path);
  unset($level1[0], $level1[1]);
  foreach ($level1 as $item) {
    if (is_dir($path .'/'. $item)) {
      $menupath = $path .'/'. $item;
      $files = scandir($menupath);
      unset($files[0], $files[1]);
      foreach($files as $filename) {
        cforge_default_content($menupath, $filename, $item);
      }
    }
    else {
      cforge_default_content($path, $item);
    }
  }
  //also make a special
  cforge_install_node_save(
    'story',
    st('Step by step guide to setting up your site'),
    cforge_cut('http://communityforge.net/'.$language->language.'/step-by-step', '<cfscrape>', '</cfscrape>'),
    FILTER_HTML_ESCAPE,
    $language->language,
    0,
    'step-by-step'
  );

  //finally import the chosen taxonomy into vocab 1
  if ($values['vocab_1'] != '<none>') {
    drupal_install_modules(array('taxonomy_xml'));
    module_load_include('module', 'taxonomy_xml');
    $options = array(
//      'vid' => db_result(db_query("SELECT vid FROM {vocabulary}  WHERE module = 'offers_wants' AND hierarchy = 1")),
      'vid' => variable_get('offers_wants_cat_vid', 0),
      'format' => 'xml',
      'recurse_down' => '',
      'taxonomy_xml_watchdog_level' => 5,
      'flush_cache' => 'Flush Cache'
    );
    drupal_set_message('taxonomy_xml_invoke_import_on_filepath exists: '. function_exists('taxonomy_xml_invoke_import_on_filepath'));
    if (function_exists('taxonomy_xml_invoke_import_on_filepath')) {
      taxonomy_xml_invoke_import_on_filepath(CFORGE_PROFILE_PATH . 'taxonomy_import/1/'.$values['vocab_1'], $options);
      drupal_uninstall_module('taxonomy_xml');
    }
    else {
      drupal_set_message(drupal_get_filename('module', 'taxonomy_xml'));
    }
  }
  //captcha settings are in cforge_custom.install
  cforge_configure_captcha();

  drupal_install_modules(array('cforge_report'));
}

function cforge_default_content($path, $file, $menuname = NULL) {
  global $language;
  list($filename, $extension) = explode('.', $file);
  switch ($extension) {
    case 'txt': $format = FILTER_HTML_ESCAPE; break;
    case 'htm': $format = FILTER_HTML_ESCAPE; break;//TODO when I'm not so tired
    default: return;
  }
  list($weight, $machine_name) = explode('_', $filename);
  $filepath = $path .'/'. $file;
  $content = file_get_contents($filepath);
  cforge_install_node_save(
    $menuname ? 'page' : 'story',
    cf_strstr($content , "\n", TRUE),//firstline of content is the title
    cf_strstr($content , "\n", FALSE),
    $format,
    $language->language,
    $weight,
    $machine_name,
    $menuname
  );
}

function cforge_install_node_save($type, $title, $body, $format, $language, $weight, $alias = '', $menuname = '') {
  $node = array(
    'type' => $type,
    'uid' => '1',
    'status' => '1',
    'comment' => '0',
    'sticky' => '0',
    'format' => $format,
    'language' => $language,
    'title' => $title,
    'body' => $body,
  );
  if ($menuname) {
    $node['menu'] = array(
      'link_title' => $title,
      'menu_name' => $menuname,
      'module' => 'menu',
      'weight' => $weight
    );
  }
  $node = (object)$node;
  node_save($node);
  if ($alias) {
    db_query("INSERT into {url_alias} (src, dst, language) VALUES ('node/%d', '%s', '%s')",
      $node->nid, $alias, $language->language);
  }
}

//configure drupal core and the modules installed in the first round
function cforge_general_setup() {
  //some defaults...before language variables installed
  variable_set('date_format_short', "d-m-y");
  variable_set('date_format_medium', "H:i, D jS M");
  variable_set('date_format_long', "H:i, l jS F, Y");

  $types = array(
    array(
      'type' => 'page',
      'name' => st('Page'),
      'module' => 'node',
      'description' => st("If you wish to add a menu item, please create a page. This is useful for adding pages such as 'our charter' that you'd like to display to visitors, or members."),
      'custom' => TRUE,
      'modified' => TRUE,
      'locked' => FALSE,
    ),
    array(
      'type' => 'story',
      'name' => st('Article'),
      'module' => 'node',
      'description' => st("Create and post articles that inform your members and animate your association. By default, the info appears on the 'Info' page and provides members can add comments."),
      'custom' => TRUE,
      'modified' => TRUE,
      'locked' => FALSE,
    ),
  );
  foreach ($types as $type) {
    $type = (object) _node_type_set_defaults($type);
    node_type_save($type);
  }
  db_query("UPDATE {node_type} SET name = '%s' where type = '%s'", array(':name'=> st('Newsletter'), ':type' => 'simplenews'));

  //create roles
  db_query("INSERT INTO {role} (rid, name)
      VALUES (%d, 'trader'), (%d, 'committee'), (%d, 'accountant'), (%d, 'local admin'), (%d, 'system');",
    array(RID_TRADER, RID_COMMITTEE, RID_ACCOUNTANT, RID_LOCAL_ADMIN, RID_SYSTEM)
  );

  //create profile fields
  $fields = array(
    1 => array('profile_firstname', st('First name'), '', st('Details'), 'textfield', -20, 1, 1, 2, 0),
    2 => array('profile_familyname', st('Family name'), '', st('Details'), 'textfield', -19, 1, 1, 2, 0),
    3 => array('profile_responsibility', st('Responsibility'), st('Job title, for committee members only'), st('Details'), 'textfield', -15, 0, 0, 2, 0),
    4 => array('profile_phone1', st('Phone Number 1'), '', st('Details'), 'textfield', 0, 0, 1, 1, 0),
    5 => array('profile_phone2', st('Phone Number 2'), '', st('Details'), 'textfield', 0, 0, 1, 1, 0),
    6 => array('profile_locality', st('Area Code'), st('This may be the zip code, or some other way to group the members geographically'), st('Details'), 'textfield', 11, 0, 1, 2, 0),
    7 => array('profile_postal', st('Postal member'), st('Please post printed directories to me '), st('Details'), 'checkbox', 21, 0, 1, 2, 0),
    8 => array('profile_address', st('Address'), st('This is necessary if you want to receive printed updates'), st('Details'), 'textarea', 10, 0, 1, 2, 0),
    9 => array('profile_notes', st('Notes'), st('This is for any additional notes.'), st('Details'), 'textarea', 50, 0, 1, 2, 0),
  );
  foreach ($fields as $fid => $values) {
    db_query("REPLACE into {profile_fields}
      (fid, name, title, explanation, category, type, weight, required, register, visibility, autocomplete)
      VALUES ($fid, '%s', '%s', '%s', '%s', '%s', %d, %d, %d, %d, %d)",
      $values
    );
  }

  //Allow committee to use html filter
  db_query("UPDATE {filter_formats} SET roles = ',%d,' WHERE name = 'Full HTML'", array(RID_COMMITTEE));

  //set theme
  variable_set('theme_default', 'sky_seldulac');
  variable_set('theme_settings', array(
    'toggle_logo' => TRUE,
    'toggle_name' => TRUE,
    'toggle_slogan' => 1,
    'toggle_mission' => 0,
    'toggle_node_user_picture' => 0,
    "toggle_comment_user_picture" => 0,
    "toggle_search" => 1,
    "toggle_favicon" => 1,
    "toggle_primary_links" => 1,
    "toggle_secondary_links" => 1,
    "toggle_node_info_event" => 1,
    "toggle_node_info_faq" => 0,
    "toggle_node_info_image" => 1,
    "toggle_node_info_story" => 0,
    "toggle_node_info_page" => 0,
    "toggle_node_info_exchange" => 1,
    "toggle_node_info_proposition" => 1,
    "toggle_node_info_document" => 1,
    "default_favicon"=> 1,
    "default_logo"=> 1,
  ));
  variable_set('theme_sky_seldulac_settings', array(
    'sky_seldulac_breadcrumbs' => 0,
    'sky_seldulac_breadcrumbs_sep' => '&raquo;',
    'sky_seldulac_font' => 'lucida',
    'sky_seldulac_font_headings' => 'lucida',
    'sky_seldulac_font_size' => '12px',
    'sky_seldulac_header_height' => 'auto',
    'sky_seldulac_layout' => 'fluid_98',
    'sky_seldulac_title' => 1,
    'sky_seldulac_nav_alignment' => 'center',
    'sky_seldulac_sub_navigation_size' => '15em',
    'sky_seldulac_wireframe' => '0',
    'sky_seldulac_themer_classes' => 0
  ));

  variable_set('image_attach_simplenews', TRUE);
  variable_set('image_attach_size_body_simplenews', 'preview');
  variable_set('image_attach_size_teaser_simplenews', 'thumbnail');
  variable_set('image_attach_story', TRUE);
  variable_set('image_attach_size_body_story', 'preview');
  variable_set('image_attach_size_teaser_story', 'thumbnail');
  variable_set('image_attach_page', TRUE);
  variable_set('image_attach_size_body_page', 'preview');
  variable_set('image_attach_size_teaser_page', 'thumbnail');
  $image_sizes = array(
    '_original' => array(
      'label' => 'Original',
      'operation' => 'scale',
      'width' => '',
      'height' => '',
      'link' => 1,
    ),
    'thumbnail' => array(
      'label' => 'Vignette',
      'operation' => 'scale',
      'width' => 150,
      'height' => 150,
      'link' => 1,
    ),
    'preview' => array(
      'label' => 'Preview',
      'operation' => 'scale',
      'width' => 640,
      'height' => 640,
      'link' => 1,
    )
  );
  variable_set('image_sizes', $image_sizes);
  db_query(
    "UPDATE {node_type} SET title_label = '%s' WHERE type = 'image'",
    array(':label' => st("An image which appears in a 'gallery'. When you upload an image you choose on the form which gallery it belongs to. Create new galleries via the manage menu."))
  );


 //image gallery
  if ($vid = variable_get('image_gallery_nav_vocabulary', 0)) {
    $term = array(
      'vid' => variable_get('image_gallery_nav_vocabulary', 0),
      'name' => st('LETS life'),
      'weight' => 3,
    );
    taxonomy_save_term($term);
  }
  else {
    watchdog('install_profile', 'no image vocabulary identifed, and default gallery not installed');
  }

  //imagecache presets
  variable_set('user_picture_imagecache_comments', 'user_image_default');
  variable_set('user_picture_imagecache_profiles', 'user_image_default');
  variable_set('user_picture_imagecache_profiles_default', 'user_image_default');
  variable_set('user_picture_imagecache_profiles_min_height', '150');
  variable_set('user_picture_imagecache_profiles_min_width', '150');
  variable_set('preprocess_css', TRUE);
  variable_set('preprocess_js', TRUE);

  //node settings
  variable_set('node_options_page', array('status'));
  

  //system settings
  variable_set('site_offline_message', st("Community Forge has taken your site offline briefly for maintenance. Thank you for your patience."));
  variable_set('site_frontpage', "home");
  variable_set('site_slogan', st("Your slogan here"));
  variable_set('anonymous', st("Visitor"));
  variable_set('user_register', "2");
  variable_set('user_pictures', "1");
  variable_set('user_picture_dimensions', '1280x1024');
  variable_set('user_picture_file_size', '1024');
  variable_set('user_picture_default', 'profiles/cforge/anonymous.jpg');

  //upload settings
  variable_set('upload_uploadsize_default', 5);
  variable_set('upload_usersize_default', 25);
  variable_set('upload_event', "1");
  variable_set('upload_community_task', "1");
  variable_set('upload_documents', "1");
  variable_set('upload_members_only', "1");
  variable_set('upload_page', "1");
  variable_set('upload_simplenews', "FALSE");
  variable_set('date_default_timezone', "7200");
  variable_set('date_default_timezone_name', "Europe/Zurich");

  if (module_exists('l10n_client')) {
    variable_set('l10n_client_server', "http://localize.drupal.org");
    variable_set('l10n_client_use_server', '4');
  }
  if (module_exists('l10n_update')) {
    variable_set('l10n_update_check_disabled', "0");
    variable_set('l10n_update_check_frequency', "1");
    variable_set('l10n_update_check_mode', "3");
    variable_set('l10n_update_download_store', "sites/all/translations");
    variable_set('l10n_update_import_mode', "0");
  }

  //comment settings
  variable_set('comment_image', 2);
  variable_set('comment_page', 2);
  variable_set('comment_story', 2);
  
  //contact settings
  $default_mail = variable_get('site_mail', 'blah@blah.com');
  db_query(
    "INSERT INTO {contact} (`category`, `recipients`, `reply`, `weight`, `selected`) VALUES
      ('%s', '$default_mail', '', 1, 1),
      ('%s', '$default_mail', '', 2, 0),
      ('%s', '$default_mail', '', 3, 0),
      ('%s', '$default_mail', '', 4, 0),
      ('%s', '$default_mail', '', 5, 0),
      ('%s', '$default_mail', '', 6, 0),
      ('%s', '$default_mail', '', 7, 0),
      ('%s', 'installations@communityforge.net', '', 9, 0),
      ('%s', 'feedback@communityforge.net', '', 10, 0)",
    array(
      ':info' => st('Information request'),
      ':mem' => st('Membership'),
      ':act' => st('Activities'),
      ':acc' => st('Accounting'),
      ':webtec' => st('Website (tech)'),
      ':webcon' => st('Website (content)'),
      ':med' => st('Mediation'),
      ':imp' => st('Suggest how to improve this site'),
      ':build' => st('Contact the builder of this site'),
    )
  );
  if ($language->language == 'fr') {
    db_query("INSERT INTO {contact} (`category`, `recipients`, `reply`, `weight`, `selected`) VALUES
      ('Route des SELs', '$default_mail', '', 8, 0),
    ");
    variable_set('date_first_day', 1);
  }
  //login destination settings
  variable_set('ld_url_destination', "news/recent");

  drupal_install_modules(array('content_copy'));
  //import the event contentType, with it's datetime field,
  //from an included file, which is a dump generated by content_copy module
  include_once './'. drupal_get_path('module', 'node') .'/content_types.inc';
  include_once('./'. drupal_get_path('module', 'content') .'/includes/content.admin.inc');
  $form_state['values']['type_name'] ='<create>';
  $form_state['values']['macro'] = file_get_contents(CFORGE_PROFILE_PATH . 'datefield.cck');
  drupal_execute("content_copy_import_form", $form_state);
  content_clear_type_cache();
}

function _cforge_filter_txtfile($string) {
  return strpos($string, '.xml') || strpos($string, '.txt');
}

function cf_strstr($haystack, $needle, $before_needle=FALSE) {
 //Find the position of $needle
 if($case_sensitive) {
  $pos=strpos($haystack,$needle);
 } else {
  $pos=strpos(strtolower($haystack),strtolower($needle));
 }
 //If $needle not found, abort
 if($pos===FALSE) return FALSE;
 //get everything from 0 to $pos?
 if($before_needle) return substr($haystack,0,$pos);
 //otherwise, go from $pos to end
 return substr($haystack,$pos);
}

function cforge_profile_drush_process() {
  $form = cforge_install_form();
  unset($form['#action'], $form['#redirect'], $form['submit']);
  $fieldnames = array_keys($form);
  $fieldnames = array_merge($fieldnames, element_children($form['local_admin']));
  unset($fieldnames['local admin']);
  drupal_set_message('Keys = ' . implode(', ', $fieldnames));
  foreach($fieldnames as $fieldname) {
    $values[$fieldname] = drush_get_option($fieldname, NULL, HOSTING_TASK);
  }
  drupal_set_message('Values = ' . implode(', ', $values));
  $form_state = array('values'=> $values);
  cforge_install_form_submit(array(), $form_state);
}

