<?php
// $Id: role_expire.rules.inc,v 1.3 2010/03/07 19:23:52 stewsnooze Exp $

/**
 * @file
 * Rules integration for the role expire module.
 */

/**
 * Implementation of hook_rules_action_info().
 */
function role_expire_rules_action_info() {
  $items['role_expire_rules_action_set_role_expire'] = array(
    'label' => t("Add expiration date/time to user's role"),
    'help' => t('Add an expiration date/time to a role a user has. This action does not add the role, only the expiration date/time.'),
    'arguments' => array(
      'user' => array(
        'type' => 'user',
        'label' => t('The user which the expiration time will be added'),
      ),
    ),
    'eval input' => array('timestamp'),
    'module' => 'Role expire',
  );

  $items['role_expire_rules_action_remove_role_expire'] = array(
    'label' => t("Remove expiration date/time from user's role"),
    'help' => t('Remove expiration date/time from a role a user has. This action does not remove the role, only the expiration date/time.'),
    'arguments' => array(
      'user' => array(
        'type' => 'user',
        'label' => t('The user which the expiration time will be removed'),
      ),
    ),
    'module' => 'Role expire',
  );

  return $items;
}

/**
 * Action: Add expire time to role.
 */
function role_expire_rules_action_set_role_expire($account, $settings) {
  role_expire_write_record($account->uid, $settings['rid'], strtotime($settings['timestamp']));
}

/**
 * Action: Add expire time to role form configuration.
 */
function role_expire_rules_action_set_role_expire_form($settings = array(), &$form) {
  $settings += array('rid' => '', 'timestamp' => '');

  _role_expire_rules_action_form_guts($form, $settings['rid'], $settings['timestamp']);
}

function role_expire_rules_action_set_role_expire_form_alter($form,$form_state, $options = array(), &$form) {
  $options += array('rid' => '', 'timestamp' => '');

  _role_expire_rules_action_form_guts($form, $options['rid'], $options['timestamp']);

}

/**
 * Helper function used by _form and _form_alter
 * @param form
 *   IN: The form we are extending.
 *   OUT: The altered form.
 * @param rid
 *   The role ID that should serve as the default value.
 * @param timestamp
 *   The date/time this role should expire
 */
function _role_expire_rules_action_form_guts(&$form, $rid, $timestamp) {

  // The role ID.
  $form['settings']['rid'] = _role_expire_get_rids($rid);

  //The timestamp.
  $form['settings']['timestamp'] = array(
    '#type' => 'textfield',
    '#title' => t('Expiration date/time'),
    '#required' => TRUE,
    '#description' => t('Enter date and time the role will expire in format <em>dd-mm-yyyy hh:mm:ss</em> or use relative times like <em>now</em>, <em>1 day</em>, <em>2 months</em>, <em>1 year</em>, <em>3 years</em>.'),
    '#default_value' => $timestamp,
  );

  // Validation
  $form['submit']['#validate'][] = '_role_expire_rules_action_validate_role_expires';
}

function _role_expire_rules_action_validate_role_expires($form,$form_state) {
  $expires = strtotime($form_state['values']['settings']['timestamp']);
  $now = time();
  if (empty($expires) || $expires < $now) {
    form_set_error('settings][timestamp','Role expiration date must be a valid future date or length of time.');
  }
}
/**
 * Action: Remove expire time from role
 */
function role_expire_rules_action_remove_role_expire($account, $settings) {
  role_expire_delete_record($account->uid, $settings['rid']);
}

/**
 * Action: Add expire time to role form configuration.
 */
function role_expire_rules_action_remove_role_expire_form($settings = array(), &$form) {
  $settings += array('rid' => '');

  // The role ID.
  $form['settings']['rid'] = _role_expire_get_rids($settings);
}


/**
 * Helper function; Return a form element with the valid roles.
 * @param default_rid
 *   The ID of the role that should be used as the default value.
 * @return unknown_type
 */
function _role_expire_get_rids($default_rid) {
  // Get valid roles.
  $roles = user_roles(TRUE);
  unset($roles[DRUPAL_AUTHENTICATED_RID]);
  // The role ID.
  return array(
    '#type' => 'select',
    '#options' => $roles,
    '#title' => t('Role'),
    '#required' => TRUE,
    '#description' => t('Select the role to affect.'),
    '#default_value' => $default_rid,
  );
}